@@ -4,20 +4,27 @@ mod intra_doc_links;
 
 use std::convert::{TryFrom, TryInto};
 
+use either::Either;
 use pulldown_cmark::{BrokenLink, CowStr, Event, InlineStr, LinkType, Options, Parser, Tag};
 use pulldown_cmark_to_cmark::{cmark_with_options, Options as CmarkOptions};
 use stdx::format_to;
 use url::Url;
 
 use hir::{
-    db::HirDatabase, Adt, AsAssocItem, AssocItem, AssocItemContainer, Crate, HasAttrs, ModuleDef,
+    db::HirDatabase, Adt, AsAssocItem, AssocItem, AssocItemContainer, Crate, HasAttrs, MacroDef,
+    ModuleDef,
 };
 use ide_db::{
     defs::{Definition, NameClass, NameRefClass},
     helpers::pick_best_token,
     RootDatabase,
 };
-use syntax::{ast, match_ast, AstNode, SyntaxKind::*, SyntaxNode, TextRange, T};
+use syntax::{
+    ast::{self, IsString},
+    match_ast, AstNode, AstToken,
+    SyntaxKind::*,
+    SyntaxNode, SyntaxToken, TextRange, TextSize, T,
+};
 
 use crate::{
     doc_links::intra_doc_links::{parse_intra_doc_link, strip_prefixes_suffixes},
@@ -47,7 +54,7 @@ pub(crate) fn rewrite_links(db: &RootDatabase, markdown: &str, definition: Defin
                 return rewritten;
             }
             if let Definition::ModuleDef(def) = definition {
-                if let Some(target) = rewrite_url_link(db, def, target) {
+                if let Some(target) = rewrite_url_link(db, Either::Left(def), target) {
                     return (target, title.to_string());
                 }
             }
@@ -169,7 +176,7 @@ pub(crate) fn resolve_doc_path_for_def(
     def: Definition,
     link: &str,
     ns: Option<hir::Namespace>,
-) -> Option<hir::ModuleDef> {
+) -> Option<Either<ModuleDef, MacroDef>> {
     match def {
         Definition::ModuleDef(def) => match def {
             hir::ModuleDef::Module(it) => it.resolve_doc_path(db, link, ns),
@@ -218,6 +225,66 @@ pub(crate) fn doc_attributes(
     }
 }
 
+pub(crate) struct DocCommentToken {
+    doc_token: SyntaxToken,
+    prefix_len: TextSize,
+}
+
+pub(crate) fn token_as_doc_comment(doc_token: &SyntaxToken) -> Option<DocCommentToken> {
+    (match_ast! {
+        match doc_token {
+            ast::Comment(comment) => TextSize::try_from(comment.prefix().len()).ok(),
+            ast::String(string) => doc_token.ancestors().find_map(ast::Attr::cast)
+                .filter(|attr| attr.simple_name().as_deref() == Some("doc")).and_then(|_| string.open_quote_text_range().map(|it| it.len())),
+            _ => None,
+        }
+    }).map(|prefix_len| DocCommentToken { prefix_len, doc_token: doc_token.clone() })
+}
+
+impl DocCommentToken {
+    pub(crate) fn get_definition_with_descend_at<T>(
+        self,
+        sema: &Semantics<RootDatabase>,
+        offset: TextSize,
+        // Definition, CommentOwner, range of intra doc link in original file
+        mut cb: impl FnMut(Definition, SyntaxNode, TextRange) -> Option<T>,
+    ) -> Option<T> {
+        let DocCommentToken { prefix_len, doc_token } = self;
+        // offset relative to the comments contents
+        let original_start = doc_token.text_range().start();
+        let relative_comment_offset = offset - original_start - prefix_len;
+
+        sema.descend_into_macros_many(doc_token).into_iter().find_map(|t| {
+            let (node, descended_prefix_len) = match_ast! {
+                match t {
+                    ast::Comment(comment) => (t.parent()?, TextSize::try_from(comment.prefix().len()).ok()?),
+                    ast::String(string) => (t.ancestors().skip_while(|n| n.kind() != ATTR).nth(1)?, string.open_quote_text_range()?.len()),
+                    _ => return None,
+                }
+            };
+            let token_start = t.text_range().start();
+            let abs_in_expansion_offset = token_start + relative_comment_offset + descended_prefix_len;
+
+            let (attributes, def) = doc_attributes(sema, &node)?;
+            let (docs, doc_mapping) = attributes.docs_with_rangemap(sema.db)?;
+            let (in_expansion_range, link, ns) =
+                extract_definitions_from_docs(&docs).into_iter().find_map(|(range, link, ns)| {
+                    let mapped = doc_mapping.map(range)?;
+                    (mapped.value.contains(abs_in_expansion_offset)).then(|| (mapped.value, link, ns))
+                })?;
+            // get the relative range to the doc/attribute in the expansion
+            let in_expansion_relative_range = in_expansion_range - descended_prefix_len - token_start;
+            // Apply relative range to the original input comment
+            let absolute_range = in_expansion_relative_range + original_start + prefix_len;
+            let def = match resolve_doc_path_for_def(sema.db, def, &link, ns)? {
+                Either::Left(it) => Definition::ModuleDef(it),
+                Either::Right(it) => Definition::Macro(it),
+            };
+            cb(def, node, absolute_range)
+        })
+    }
+}
+
 fn broken_link_clone_cb<'a, 'b>(link: BrokenLink<'a>) -> Option<(CowStr<'b>, CowStr<'b>)> {
     // These allocations are actually unnecessary but the lifetimes on BrokenLinkCallback are wrong
     // this is fixed in the repo but not on the crates.io release yet
@@ -243,9 +310,9 @@ fn get_doc_link(db: &RootDatabase, definition: Definition) -> Option<String> {
                     AssocItemContainer::Impl(i) => i.self_ty(db).as_adt()?.into(),
                 };
                 let frag = get_assoc_item_fragment(db, assoc_item)?;
-                (def, Some(frag))
+                (Either::Left(def), Some(frag))
             } else {
-                (def, None)
+                (Either::Left(def), None)
             }
         }
         Definition::Field(field) => {
@@ -254,10 +321,9 @@ fn get_doc_link(db: &RootDatabase, definition: Definition) -> Option<String> {
                 hir::VariantDef::Union(it) => it.into(),
                 hir::VariantDef::Variant(it) => it.into(),
             };
-            (def, Some(format!("structfield.{}", field.name(db))))
+            (Either::Left(def), Some(format!("structfield.{}", field.name(db))))
         }
-        // FIXME macros
-        Definition::Macro(_) => return None,
+        Definition::Macro(makro) => (Either::Right(makro), None),
         // FIXME impls
         Definition::SelfType(_) => return None,
         Definition::Local(_) | Definition::GenericParam(_) | Definition::Label(_) => return None,
@@ -270,7 +336,7 @@ fn get_doc_link(db: &RootDatabase, definition: Definition) -> Option<String> {
         url = url.join(&path).ok()?;
     }
 
-    url = url.join(&get_symbol_filename(db, &target)?).ok()?;
+    url = url.join(&get_symbol_filename(db, target)?).ok()?;
     url.set_fragment(frag.as_deref());
 
     Some(url.into())
@@ -292,24 +358,29 @@ fn rewrite_intra_doc_link(
         url = url.join(&path).ok()?;
     }
 
-    let (resolved, frag) = if let Some(assoc_item) = resolved.as_assoc_item(db) {
-        let resolved = match assoc_item.container(db) {
-            AssocItemContainer::Trait(t) => t.into(),
-            AssocItemContainer::Impl(i) => i.self_ty(db).as_adt()?.into(),
+    let (resolved, frag) =
+        if let Some(assoc_item) = resolved.left().and_then(|it| it.as_assoc_item(db)) {
+            let resolved = match assoc_item.container(db) {
+                AssocItemContainer::Trait(t) => t.into(),
+                AssocItemContainer::Impl(i) => i.self_ty(db).as_adt()?.into(),
+            };
+            let frag = get_assoc_item_fragment(db, assoc_item)?;
+            (Either::Left(resolved), Some(frag))
+        } else {
+            (resolved, None)
         };
-        let frag = get_assoc_item_fragment(db, assoc_item)?;
-        (resolved, Some(frag))
-    } else {
-        (resolved, None)
-    };
-    url = url.join(&get_symbol_filename(db, &resolved)?).ok()?;
+    url = url.join(&get_symbol_filename(db, resolved)?).ok()?;
     url.set_fragment(frag.as_deref());
 
     Some((url.into(), strip_prefixes_suffixes(title).to_string()))
 }
 
 /// Try to resolve path to local documentation via path-based links (i.e. `../gateway/struct.Shard.html`).
-fn rewrite_url_link(db: &RootDatabase, def: ModuleDef, target: &str) -> Option<String> {
+fn rewrite_url_link(
+    db: &RootDatabase,
+    def: Either<ModuleDef, MacroDef>,
+    target: &str,
+) -> Option<String> {
     if !(target.contains('#') || target.contains(".html")) {
         return None;
     }
@@ -321,25 +392,35 @@ fn rewrite_url_link(db: &RootDatabase, def: ModuleDef, target: &str) -> Option<S
         url = url.join(&path).ok()?;
     }
 
-    url = url.join(&get_symbol_filename(db, &def)?).ok()?;
+    url = url.join(&get_symbol_filename(db, def)?).ok()?;
     url.join(target).ok().map(Into::into)
 }
 
-fn crate_of_def(db: &RootDatabase, def: ModuleDef) -> Option<Crate> {
+fn crate_of_def(db: &RootDatabase, def: Either<ModuleDef, MacroDef>) -> Option<Crate> {
     let krate = match def {
         // Definition::module gives back the parent module, we don't want that as it fails for root modules
-        ModuleDef::Module(module) => module.krate(),
-        _ => def.module(db)?.krate(),
+        Either::Left(ModuleDef::Module(module)) => module.krate(),
+        Either::Left(def) => def.module(db)?.krate(),
+        Either::Right(def) => def.module(db)?.krate(),
     };
     Some(krate)
 }
 
-fn mod_path_of_def(db: &RootDatabase, def: ModuleDef) -> Option<String> {
-    def.canonical_module_path(db).map(|it| {
-        let mut path = String::new();
-        it.flat_map(|it| it.name(db)).for_each(|name| format_to!(path, "{}/", name));
-        path
-    })
+fn mod_path_of_def(db: &RootDatabase, def: Either<ModuleDef, MacroDef>) -> Option<String> {
+    match def {
+        Either::Left(def) => def.canonical_module_path(db).map(|it| {
+            let mut path = String::new();
+            it.flat_map(|it| it.name(db)).for_each(|name| format_to!(path, "{}/", name));
+            path
+        }),
+        Either::Right(def) => {
+            def.module(db).map(|it| it.path_to_root(db).into_iter().rev()).map(|it| {
+                let mut path = String::new();
+                it.flat_map(|it| it.name(db)).for_each(|name| format_to!(path, "{}/", name));
+                path
+            })
+        }
+    }
 }
 
 /// Rewrites a markdown document, applying 'callback' to each link.
@@ -405,27 +486,34 @@ fn get_doc_base_url(db: &RootDatabase, krate: &Crate) -> Option<Url> {
 /// https://doc.rust-lang.org/std/iter/trait.Iterator.html#tymethod.next
 ///                                    ^^^^^^^^^^^^^^^^^^^
 /// ```
-fn get_symbol_filename(db: &dyn HirDatabase, definition: &ModuleDef) -> Option<String> {
-    Some(match definition {
-        ModuleDef::Adt(adt) => match adt {
-            Adt::Struct(s) => format!("struct.{}.html", s.name(db)),
-            Adt::Enum(e) => format!("enum.{}.html", e.name(db)),
-            Adt::Union(u) => format!("union.{}.html", u.name(db)),
+fn get_symbol_filename(
+    db: &dyn HirDatabase,
+    definition: Either<ModuleDef, MacroDef>,
+) -> Option<String> {
+    let res = match definition {
+        Either::Left(definition) => match definition {
+            ModuleDef::Adt(adt) => match adt {
+                Adt::Struct(s) => format!("struct.{}.html", s.name(db)),
+                Adt::Enum(e) => format!("enum.{}.html", e.name(db)),
+                Adt::Union(u) => format!("union.{}.html", u.name(db)),
+            },
+            ModuleDef::Module(m) => match m.name(db) {
+                Some(name) => format!("{}/index.html", name),
+                None => String::from("index.html"),
+            },
+            ModuleDef::Trait(t) => format!("trait.{}.html", t.name(db)),
+            ModuleDef::TypeAlias(t) => format!("type.{}.html", t.name(db)),
+            ModuleDef::BuiltinType(t) => format!("primitive.{}.html", t.name()),
+            ModuleDef::Function(f) => format!("fn.{}.html", f.name(db)),
+            ModuleDef::Variant(ev) => {
+                format!("enum.{}.html#variant.{}", ev.parent_enum(db).name(db), ev.name(db))
+            }
+            ModuleDef::Const(c) => format!("const.{}.html", c.name(db)?),
+            ModuleDef::Static(s) => format!("static.{}.html", s.name(db)?),
         },
-        ModuleDef::Module(m) => match m.name(db) {
-            Some(name) => format!("{}/index.html", name),
-            None => String::from("index.html"),
-        },
-        ModuleDef::Trait(t) => format!("trait.{}.html", t.name(db)),
-        ModuleDef::TypeAlias(t) => format!("type.{}.html", t.name(db)),
-        ModuleDef::BuiltinType(t) => format!("primitive.{}.html", t.name()),
-        ModuleDef::Function(f) => format!("fn.{}.html", f.name(db)),
-        ModuleDef::Variant(ev) => {
-            format!("enum.{}.html#variant.{}", ev.parent_enum(db).name(db), ev.name(db))
-        }
-        ModuleDef::Const(c) => format!("const.{}.html", c.name(db)?),
-        ModuleDef::Static(s) => format!("static.{}.html", s.name(db)?),
-    })
+        Either::Right(mac) => format!("macro.{}.html", mac.name(db)?),
+    };
+    Some(res)
 }
 
 /// Get the fragment required to link to a specific field, method, associated type, or associated constant.
