@@ -1,13 +1,13 @@
 use std::convert::TryFrom;
 
 use ide_assists::utils::extract_trivial_expression;
+use ide_db::helpers::node_ext::expr_as_name_ref;
 use itertools::Itertools;
 use syntax::{
-    algo::non_trivia_sibling,
     ast::{self, AstNode, AstToken, IsString},
-    Direction, NodeOrToken, SourceFile, SyntaxElement,
+    NodeOrToken, SourceFile, SyntaxElement,
     SyntaxKind::{self, USE_TREE, WHITESPACE},
-    SyntaxNode, SyntaxToken, TextRange, TextSize, T,
+    SyntaxToken, TextRange, TextSize, T,
 };
 
 use text_edit::{TextEdit, TextEditBuilder};
@@ -16,6 +16,7 @@ pub struct JoinLinesConfig {
     pub join_else_if: bool,
     pub remove_trailing_comma: bool,
     pub unwrap_trivial_blocks: bool,
+    pub join_assignments: bool,
 }
 
 // Feature: Join Lines
@@ -135,10 +136,9 @@ fn remove_newline(
             }
             T!['}'] => {
                 // Removes: comma, newline (incl. surrounding whitespace)
-                let space = if let Some(left) = prev.prev_sibling_or_token() {
-                    compute_ws(left.kind(), next.kind())
-                } else {
-                    " "
+                let space = match prev.prev_sibling_or_token() {
+                    Some(left) => compute_ws(left.kind(), next.kind()),
+                    None => " ",
                 };
                 edit.replace(
                     TextRange::new(prev.text_range().start(), token.text_range().end()),
@@ -163,6 +163,12 @@ fn remove_newline(
         }
     }
 
+    if config.join_assignments {
+        if join_assignments(edit, &prev, &next).is_some() {
+            return;
+        }
+    }
+
     if config.unwrap_trivial_blocks {
         // Special case that turns something like:
         //
@@ -204,15 +210,8 @@ fn remove_newline(
     edit.replace(token.text_range(), compute_ws(prev.kind(), next.kind()).to_string());
 }
 
-fn has_comma_after(node: &SyntaxNode) -> bool {
-    match non_trivia_sibling(node.clone().into(), Direction::Next) {
-        Some(n) => n.kind() == T![,],
-        _ => false,
-    }
-}
-
 fn join_single_expr_block(edit: &mut TextEditBuilder, token: &SyntaxToken) -> Option<()> {
-    let block_expr = ast::BlockExpr::cast(token.parent()?)?;
+    let block_expr = ast::BlockExpr::cast(token.ancestors().nth(1)?)?;
     if !block_expr.is_standalone() {
         return None;
     }
@@ -223,7 +222,7 @@ fn join_single_expr_block(edit: &mut TextEditBuilder, token: &SyntaxToken) -> Op
 
     // Match block needs to have a comma after the block
     if let Some(match_arm) = block_expr.syntax().parent().and_then(ast::MatchArm::cast) {
-        if !has_comma_after(match_arm.syntax()) {
+        if match_arm.comma_token().is_none() {
             buf.push(',');
         }
     }
@@ -240,6 +239,41 @@ fn join_single_use_tree(edit: &mut TextEditBuilder, token: &SyntaxToken) -> Opti
     Some(())
 }
 
+fn join_assignments(
+    edit: &mut TextEditBuilder,
+    prev: &SyntaxElement,
+    next: &SyntaxElement,
+) -> Option<()> {
+    let let_stmt = ast::LetStmt::cast(prev.as_node()?.clone())?;
+    if let_stmt.eq_token().is_some() {
+        cov_mark::hit!(join_assignments_already_initialized);
+        return None;
+    }
+    let let_ident_pat = match let_stmt.pat()? {
+        ast::Pat::IdentPat(it) => it,
+        _ => return None,
+    };
+
+    let expr_stmt = ast::ExprStmt::cast(next.as_node()?.clone())?;
+    let bin_expr = match expr_stmt.expr()? {
+        ast::Expr::BinExpr(it) => it,
+        _ => return None,
+    };
+    if !matches!(bin_expr.op_kind()?, ast::BinaryOp::Assignment { op: None }) {
+        return None;
+    }
+    let lhs = bin_expr.lhs()?;
+    let name_ref = expr_as_name_ref(&lhs)?;
+
+    if name_ref.to_string() != let_ident_pat.syntax().to_string() {
+        cov_mark::hit!(join_assignments_mismatch);
+        return None;
+    }
+
+    edit.delete(let_stmt.semicolon_token()?.text_range().cover(lhs.syntax().text_range()));
+    Some(())
+}
+
 fn as_if_expr(element: &SyntaxElement) -> Option<ast::IfExpr> {
     let mut node = element.as_node()?.clone();
     if let Some(stmt) = ast::ExprStmt::cast(node.clone()) {
@@ -283,6 +317,7 @@ mod tests {
             join_else_if: true,
             remove_trailing_comma: true,
             unwrap_trivial_blocks: true,
+            join_assignments: true,
         };
 
         let (before_cursor_pos, before) = extract_offset(ra_fixture_before);
@@ -308,6 +343,7 @@ mod tests {
             join_else_if: true,
             remove_trailing_comma: true,
             unwrap_trivial_blocks: true,
+            join_assignments: true,
         };
 
         let (sel, before) = extract_range(ra_fixture_before);
@@ -998,6 +1034,55 @@ fn main() {
 
     }
 }
+"#,
+        );
+    }
+
+    #[test]
+    fn join_assignments() {
+        check_join_lines(
+            r#"
+fn foo() {
+    $0let foo;
+    foo = "bar";
+}
+"#,
+            r#"
+fn foo() {
+    $0let foo = "bar";
+}
+"#,
+        );
+
+        cov_mark::check!(join_assignments_mismatch);
+        check_join_lines(
+            r#"
+fn foo() {
+    let foo;
+    let qux;$0
+    foo = "bar";
+}
+"#,
+            r#"
+fn foo() {
+    let foo;
+    let qux;$0 foo = "bar";
+}
+"#,
+        );
+
+        cov_mark::check!(join_assignments_already_initialized);
+        check_join_lines(
+            r#"
+fn foo() {
+    let foo = "bar";$0
+    foo = "bar";
+}
+"#,
+            r#"
+fn foo() {
+    let foo = "bar";$0 foo = "bar";
+}
 "#,
         );
     }
